import bpy
from bpy.types import AddonPreferences
from bpy.props import EnumProperty, BoolProperty

def get_key_items():
    """返回所有可用的快捷键选项"""
    # 鼠标按键
    mouse_buttons = [
        ('LEFTMOUSE', "Left Mouse", "Left mouse button"),
        ('MIDDLEMOUSE', "Middle Mouse", "Middle mouse button"),
        ('RIGHTMOUSE', "Right Mouse", "Right mouse button"),
        ('BUTTON4MOUSE', "Mouse Button 4", "Mouse side button 4"),
        ('BUTTON5MOUSE', "Mouse Button 5", "Mouse side button 5"),
        ('BUTTON6MOUSE', "Mouse Button 6", "Mouse side button 6"),
        ('BUTTON7MOUSE', "Mouse Button 7", "Mouse side button 7"),
    ]
    
    # 常用键盘按键
    keyboard_keys = [
        ('SPACE', "Spacebar", "Spacebar key"),
        ('TAB', "Tab", "Tab key"),
        ('A', "A", "A key"),
        ('B', "B", "B key"),
        ('C', "C", "C key"),
        ('D', "D", "D key"),
        ('E', "E", "E key"),
        ('F', "F", "F key"),
        ('G', "G", "G key"),
        ('Q', "Q", "Q key"),
        ('R', "R", "R key"),
        ('S', "S", "S key"),
        ('W', "W", "W key"),
        ('X', "X", "X key"),
        ('Y', "Y", "Y key"),
        ('Z', "Z", "Z key"),
        ('F1', "F1", "F1 key"),
        ('F2', "F2", "F2 key"),
        ('F3', "F3", "F3 key"),
        ('F4', "F4", "F4 key"),
        ('F5', "F5", "F5 key"),
        ('F6', "F6", "F6 key"),
        ('F7', "F7", "F7 key"),
        ('F8', "F8", "F8 key"),
        ('F9', "F9", "F9 key"),
        ('F10', "F10", "F10 key"),
        ('F11', "F11", "F11 key"),
        ('F12', "F12", "F12 key"),
    ]
    
    return [
        ("MOUSE", "Mouse Buttons", ""),
        *mouse_buttons,
        ("KEYBOARD", "Keyboard Keys", ""),
        *keyboard_keys,
    ]

def update_keybind(self, context):
    """当快捷键设置改变时更新快捷键绑定"""
    # 移除现有的快捷键
    for km, kmi in addon_keymaps:
        km.keymap_items.remove(kmi)
    addon_keymaps.clear()
    
    # 重新注册快捷键
    wm = context.window_manager
    kc = wm.keyconfigs.addon
    if kc:
        # 注册快速面板快捷键
        km = kc.keymaps.new(name='3D View', space_type='VIEW_3D')
        kmi = km.keymap_items.new(
            LIGHTING_GADGETS_OT_QUICK_PANEL.bl_idname,
            type=self.quick_panel_key,
            value='PRESS',
            ctrl=self.quick_panel_ctrl,
            shift=self.quick_panel_shift,
            alt=self.quick_panel_alt
        )
        addon_keymaps.append((km, kmi))
        
        # 注册反射模式快捷键
        km = kc.keymaps.new(name='3D View', space_type='VIEW_3D')
        kmi = km.keymap_items.new(
            LIGHTING_GADGETS_OT_REFLECTION_MODE.bl_idname,
            type=self.reflection_key,
            value='PRESS',
            ctrl=self.reflection_ctrl,
            shift=self.reflection_shift,
            alt=self.reflection_alt
        )
        addon_keymaps.append((km, kmi))

class QUICK_STUDIO_LIGHT_preferences(AddonPreferences):
    bl_idname = __package__

    # 快速面板快捷键设置
    quick_panel_key: EnumProperty(
        name="Quick Panel Key",
        description="Select the key for opening the quick panel",
        items=get_key_items(),
        default='RIGHTMOUSE',
        update=update_keybind
    )
    quick_panel_ctrl: BoolProperty(
        name="Ctrl",
        description="Use Ctrl modifier for quick panel shortcut",
        default=True,
        update=update_keybind
    )
    quick_panel_shift: BoolProperty(
        name="Shift",
        description="Use Shift modifier for quick panel shortcut",
        default=False,
        update=update_keybind
    )
    quick_panel_alt: BoolProperty(
        name="Alt",
        description="Use Alt modifier for quick panel shortcut",
        default=False,
        update=update_keybind
    )

    # 反射模式快捷键设置
    reflection_key: EnumProperty(
        name="Reflection Mode Key",
        description="Select the key for reflection mode",
        items=get_key_items(),
        default='R',
        update=update_keybind
    )
    reflection_ctrl: BoolProperty(
        name="Ctrl",
        description="Use Ctrl modifier for reflection mode shortcut",
        default=False,
        update=update_keybind
    )
    reflection_shift: BoolProperty(
        name="Shift",
        description="Use Shift modifier for reflection mode shortcut",
        default=True,
        update=update_keybind
    )
    reflection_alt: BoolProperty(
        name="Alt",
        description="Use Alt modifier for reflection mode shortcut",
        default=False,
        update=update_keybind
    )

    # 界面显示设置
    border_thickness: bpy.props.FloatProperty(
        name="Border Thickness",
        description="Thickness of the viewport border",
        default=3.0,
        min=1.0,
        max=15.0,
        subtype='PIXEL'
    )
    
    text_size: bpy.props.IntProperty(
        name="Text Size",
        description="Size of the mode text",
        default=14,
        min=6,
        max=72
    )
    
    border_opacity: bpy.props.FloatProperty(
        name="Border Opacity",
        description="Opacity of the viewport border",
        default=0.8,
        min=0.05,
        max=1.0,
        subtype='FACTOR'
    )
    
    text_opacity: bpy.props.FloatProperty(
        name="Text Opacity",
        description="Opacity of the mode text",
        default=0.8,
        min=0.05,
        max=1.0,
        subtype='FACTOR'
    )
    
    # 反射模式颜色
    reflection_color: bpy.props.FloatVectorProperty(
        name="Reflection Mode Color",
        description="Color for reflection mode UI elements",
        default=(0.0, 0.5, 1.0),
        min=0.0,
        max=1.0,
        size=3,
        subtype='COLOR'
    )
    
    # 精确模式颜色
    precise_color: bpy.props.FloatVectorProperty(
        name="Precise Mode Color",
        description="Color for precise mode UI elements",
        default=(1.0, 0.5, 0.0),
        min=0.0,
        max=1.0,
        size=3,
        subtype='COLOR'
    )

    def draw(self, context):
        layout = self.layout
        
        # 快速面板快捷键设置
        box = layout.box()
        box.label(text="Quick Panel Shortcut Settings:")
        
        col = box.column()
        row = col.row()
        row.label(text="Key:")
        row.prop(self, "quick_panel_key", text="")
        
        row = col.row(align=True)
        row.label(text="Modifiers:")
        row.prop(self, "quick_panel_ctrl", toggle=True)
        row.prop(self, "quick_panel_shift", toggle=True)
        row.prop(self, "quick_panel_alt", toggle=True)
        
        box.separator()
        box.label(text="Current Shortcut: " + self.get_shortcut_text("quick_panel"))
        
        # 反射模式快捷键设置
        box = layout.box()
        box.label(text="Reflection Mode Shortcut Settings:")
        
        col = box.column()
        row = col.row()
        row.label(text="Key:")
        row.prop(self, "reflection_key", text="")
        
        row = col.row(align=True)
        row.label(text="Modifiers:")
        row.prop(self, "reflection_ctrl", toggle=True)
        row.prop(self, "reflection_shift", toggle=True)
        row.prop(self, "reflection_alt", toggle=True)
        
        box.separator()
        box.label(text="Current Shortcut: " + self.get_shortcut_text("reflection"))
        
        # 界面显示设置
        box = layout.box()
        box.label(text="UI Display Settings:")
        
        # 尺寸设置
        col = box.column(align=True)
        col.label(text="Size Settings:")
        col.prop(self, "border_thickness")
        col.prop(self, "text_size")
        
        # 透明度设置
        col = box.column(align=True)
        col.label(text="Opacity Settings:")
        col.prop(self, "border_opacity")
        col.prop(self, "text_opacity")
        
        # 颜色设置
        box = layout.box()
        box.label(text="Color Settings:")
        
        col = box.column(align=True)
        col.prop(self, "reflection_color")
        col.prop(self, "precise_color")
        
    def get_shortcut_text(self, shortcut_type):
        """返回当前快捷键组合的文本描述"""
        if shortcut_type == "quick_panel":
            key = self.quick_panel_key
            ctrl = self.quick_panel_ctrl
            shift = self.quick_panel_shift
            alt = self.quick_panel_alt
        else:  # reflection
            key = self.reflection_key
            ctrl = self.reflection_ctrl
            shift = self.reflection_shift
            alt = self.reflection_alt
            
        modifiers = []
        if ctrl:
            modifiers.append("Ctrl")
        if shift:
            modifiers.append("Shift")
        if alt:
            modifiers.append("Alt")
            
        # 获取按键的显示名称
        for item in get_key_items():
            if item[0] == key:
                key_text = item[1]
                break
        
        return " + ".join(modifiers + [key_text]) if modifiers else key_text

from .lighting_gadgets_ops import (
    LIGHTING_GADGETS_PT_QUICK_PANEL,
    LIGHTING_GADGETS_OT_QUICK_PANEL,
    LIGHTING_GADGETS_OT_REFLECTION_MODE
)

classes = (
    LIGHTING_GADGETS_PT_QUICK_PANEL,
    LIGHTING_GADGETS_OT_QUICK_PANEL,
    LIGHTING_GADGETS_OT_REFLECTION_MODE,
    QUICK_STUDIO_LIGHT_preferences,
)

addon_keymaps = []

def register():
    """Register the addon"""
    # Register classes
    for cls in classes:
        bpy.utils.register_class(cls)
        
    # Register custom properties
    bpy.types.Object.last_hit_location = bpy.props.FloatVectorProperty(
        name="Last Hit Location",
        description="Last position where the ray hit",
        subtype='XYZ',
        size=3
    )
    bpy.types.Object.last_hit_distance = bpy.props.FloatProperty(
        name="Last Hit Distance",
        description="Distance of the last ray hit"
    )
    bpy.types.Object.last_hit_normal = bpy.props.FloatVectorProperty(
        name="Last Hit Normal",
        description="Normal vector of the last ray hit surface",
        subtype='XYZ',
        size=3
    )
    
    # Register keymaps
    wm = bpy.context.window_manager
    kc = wm.keyconfigs.addon
    if kc:
        # 获取插件首选项
        try:
            prefs = bpy.context.preferences.addons[__package__].preferences
            
            # 注册快速面板快捷键
            km = kc.keymaps.new(name='3D View', space_type='VIEW_3D')
            kmi = km.keymap_items.new(
                LIGHTING_GADGETS_OT_QUICK_PANEL.bl_idname,
                type=prefs.quick_panel_key,
                value='PRESS',
                ctrl=prefs.quick_panel_ctrl,
                shift=prefs.quick_panel_shift,
                alt=prefs.quick_panel_alt
            )
            addon_keymaps.append((km, kmi))
            
            # 注册反射模式快捷键
            km = kc.keymaps.new(name='3D View', space_type='VIEW_3D')
            kmi = km.keymap_items.new(
                LIGHTING_GADGETS_OT_REFLECTION_MODE.bl_idname,
                type=prefs.reflection_key,
                value='PRESS',
                ctrl=prefs.reflection_ctrl,
                shift=prefs.reflection_shift,
                alt=prefs.reflection_alt
            )
            addon_keymaps.append((km, kmi))
            
        except (KeyError, AttributeError):
            # 如果首选项还未创建，使用默认值
            # 快速面板默认快捷键
            km = kc.keymaps.new(name='3D View', space_type='VIEW_3D')
            kmi = km.keymap_items.new(
                LIGHTING_GADGETS_OT_QUICK_PANEL.bl_idname,
                type='RIGHTMOUSE',
                value='PRESS',
                ctrl=True
            )
            addon_keymaps.append((km, kmi))
            
            # 反射模式默认快捷键
            km = kc.keymaps.new(name='3D View', space_type='VIEW_3D')
            kmi = km.keymap_items.new(
                LIGHTING_GADGETS_OT_REFLECTION_MODE.bl_idname,
                type='R',
                value='PRESS',
                shift=True
            )
            addon_keymaps.append((km, kmi))

def unregister():
    """Unregister the addon"""
    # Unregister keymaps
    for km, kmi in addon_keymaps:
        km.keymap_items.remove(kmi)
    addon_keymaps.clear()
    
    # Unregister classes
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
    
    # Remove custom properties
    del bpy.types.Object.last_hit_location
    del bpy.types.Object.last_hit_distance
    del bpy.types.Object.last_hit_normal

if __name__ == "__main__":
    register()